@@ -1,7 +1,7 @@
 // padthv1_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2021, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2022, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -685,6 +685,14 @@ void padthv1_jack::shutdown_close (void)
 #include <QApplication>
 #include <QTextStream>
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
+#include <QCommandLineParser>
+#include <QCommandLineOption>
+#if defined(Q_OS_WINDOWS)
+#include <QMessageBox>
+#endif
+#endif
+
 #ifdef CONFIG_NSM
 #include "padthv1_nsm.h"
 #endif
@@ -732,7 +740,7 @@ padthv1_jack_application::padthv1_jack_application ( int& argc, char **argv )
 	}
 
 	if (m_bGui) {
-	#if defined(Q_OS_LINUX)
+	#if defined(Q_OS_LINUX) && !defined(CONFIG_WAYLAND)
 		::setenv("QT_QPA_PLATFORM", "xcb", 0);
 	#endif
 	#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
@@ -744,6 +752,10 @@ padthv1_jack_application::padthv1_jack_application ( int& argc, char **argv )
 	#if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
 		pApp->setApplicationDisplayName(PADTHV1_TITLE);
 		//	PADTHV1_TITLE " - " + QObject::tr(PADTHV1_SUBTITLE));
+	#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
+		pApp->setDesktopFileName(
+			QString("org.rncbc.%1").arg(PACKAGE_TARNAME));
+	#endif
 	#endif
 		m_pApp = pApp;
 	} else {
@@ -752,6 +764,13 @@ padthv1_jack_application::padthv1_jack_application ( int& argc, char **argv )
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
 	m_pApp->setApplicationName(PADTHV1_TITLE);
+	QString sVersion(CONFIG_BUILD_VERSION);
+	sVersion += '\n';
+	sVersion += QString("Qt: %1").arg(qVersion());
+#if defined(QT_STATIC)
+	sVersion += "-static";
+#endif
+	m_pApp->setApplicationVersion(sVersion);
 #endif
 
 #ifdef HAVE_SIGNAL_H
@@ -809,11 +828,64 @@ padthv1_jack_application::~padthv1_jack_application (void)
 }
 
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
+
+void padthv1_jack_application::show_error( const QString& msg )
+{
+#if defined(Q_OS_WINDOWS)
+	QMessageBox::information(nullptr, QApplication::applicationName(), msg);
+#else
+	const QByteArray tmp = msg.toUtf8() + '\n';
+	::fputs(tmp.constData(), stderr);
+#endif
+}
+
+#endif
+
+
 // Argument parser method.
 bool padthv1_jack_application::parse_args (void)
 {
-	QTextStream out(stderr);
 	const QStringList& args = m_pApp->arguments();
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
+
+	QCommandLineParser parser;
+	parser.setApplicationDescription(
+		PADTHV1_TITLE " - " + QObject::tr(PADTHV1_SUBTITLE));
+
+	parser.addOption({{"g", "no-gui"},
+		QObject::tr("Disable the graphical user interface (GUI)")});
+	parser.addOption({{"n", "client-name"},
+		QObject::tr("Set the JACK client name (default: %1)")
+			.arg(PADTHV1_TITLE), "label"});
+	parser.addHelpOption();
+	parser.addVersionOption();
+	parser.addPositionalArgument("preset-file",
+		QObject::tr("Load preset file (.%1)").arg(PADTHV1_TITLE),
+		QObject::tr("[preset-file]"));
+	parser.process(args);
+
+	if (parser.isSet("no-gui")) {
+		// Ignored: parsed on startup...
+	}
+
+	if (parser.isSet("client-name")) {
+		const QString& sVal = parser.value("client-name");
+		if (sVal.isEmpty()) {
+			show_error(QObject::tr("Option -n requires an argument (label)."));
+			return false;
+		}
+		m_sClientName = sVal;
+	}
+
+	foreach(const QString& sArg, parser.positionalArguments()) {
+		m_presets.append(sArg);
+	}
+
+#else
+
+	QTextStream out(stderr);
 	const int argc = args.count();
 
 	for (int i = 1; i < argc; ++i) {
@@ -834,7 +906,7 @@ bool padthv1_jack_application::parse_args (void)
 
 		if (sArg == "-n" || sArg == "--client-name") {
 			if (sVal.isNull()) {
-				out << QObject::tr("Option -n requires an argument (client-name).\n\n");
+				out << QObject::tr("Option -n requires an argument (label).\n\n");
 				return false;
 			}
 			m_sClientName = sVal;
@@ -843,15 +915,19 @@ bool padthv1_jack_application::parse_args (void)
 		}
 		else
 		if (sArg == "-h" || sArg == "--help") {
-			out << QObject::tr(
-				"Usage: %1 [options] [preset-file]\n\n"
-				PADTHV1_TITLE " - " PADTHV1_SUBTITLE "\n\n"
-				"Options:\n\n"
-				"  -g, --no-gui\n\tDisable the graphical user interface (GUI)\n\n"
-				"  -n, --client-name=[label]\n\tSet the JACK client name (default: padthv1)\n\n"
-				"  -h, --help\n\tShow help about command line options\n\n"
-				"  -v, --version\n\tShow version information\n\n")
-				.arg(args.at(0));
+			const QString sEot = "\n\t";
+			const QString sEol = "\n\n";
+			out << QObject::tr("Usage: %1 [options]").arg(args.at(0)) + sEol;
+			out << PADTHV1_TITLE " - " << QObject::tr(PADTHV1_SUBTITLE) + sEol;
+			out << QObject::tr("Options:") + sEol;
+			out << "  -g, --no-gui" + sEot +
+				QObject::tr("Disable the graphical user interface (GUI)") + sEol;
+			out << "  -n, --client-name=[label]" + sEot +
+				QObject::tr("Set the JACK client name (default: %1)").arg(PADTHV1_TITLE) + sEol;
+			out << "  -h, --help" + sEot +
+				QObject::tr("Show help about command line options.") + sEol;
+			out << "  -v, --version" + sEot +
+				QObject::tr("Show version information.") + sEol;
 			return false;
 		}
 		else
@@ -873,6 +949,8 @@ bool padthv1_jack_application::parse_args (void)
 		}
 	}
 
+#endif
+
 	return true;
 }
 
